import 'dart:io';

import './request_handler.dart';
import './response.dart';

class App {
  final List<RequestHandler> registry = [];

  RequestHandler get(String url, Response Function(HttpRequest) handler) {
    final requestHandler = RequestHandler(
      url: url,
      handler: handler,
      method: 'GET',
    );
    registry.add(requestHandler);
    return requestHandler;
  }

  RequestHandler post(String url, Response Function(HttpRequest) handler) {
    final requestHandler = RequestHandler(
      url: url,
      handler: handler,
      method: 'POST',
    );
    registry.add(requestHandler);
    return requestHandler;
  }

  void start() async {
    final server = await HttpServer.bind(
      InternetAddress.loopbackIPv4,
      8080,
    );

    print('Spruce development server listening on port 8080.');
    print('URL - http://localhost:8080/');

    await for (HttpRequest request in server) {
      if (request.uri.path.startsWith('/styles/')) {
        final path = request.uri.path;
        final file = File('../assets/styles/${path.substring(9)}');

        try {
          final contents = file.readFileSync();
        } catch (exc) {
          request.response
            ..statusCode = 500
            ..headers.contentType = ContentType('text', 'plain')
            ..writeln('Internal Server Error 500.')
            ..close();
          continue;
        }

        request.response
          ..statusCode = 200
          ..headers.contentType = ContentType('text', 'css')
          ..writeln(contents)
          ..close();

        continue;
      }

      if (request.uri.path.startsWith('/scripts/')) {
        final path = request.uri.path;
        final file = File('../assets/scripts/${path.substring(10)}');

        try {
          final contents = file.readFileSync();
        } catch (exc) {
          request.response
            ..statusCode = 500
            ..headers.contentType = ContentType('text', 'plain')
            ..writeln('Internal Server Error 500.')
            ..close();
          continue;
        }

        request.response
          ..statusCode = 200
          ..headers.contentType = ContentType('application', 'js')
          ..writeln(contents)
          ..close();

        continue;
      }

      final handler = registry.firstWhere((handler) {
        return handler.url == request.uri.path &&
            handler.method == request.method;
      }, orElse: () {
          request.response
            ..statusCode = 404
            ..headers.contentType = ContentType('text', 'plain');
            ..writeln('404 Page Not Found.')
            ..close();
          return null;
      });
      if (handler == null) {
        continue;
      }
      final response = handler.run(request);

      response.output();
    }
  }
}
