import 'dart:io';

import './middleware.dart';
import './response.dart';
import './responses/method_not_allowed_response.dart';

class RequestHandler {
  final Response Function(HttpRequest) handler;
  final String url;
  final String method;
  final List<Middleware> middlewares = [];

  RequestHandler({
    handler,
    url,
    method,
  }) {
    this.handler = handler;
    this.url = url;
    this.method = method;

    middlewares.add(
      Middleware(
        (request, next) {
          if (request.method != this.method) {
            return MethodNotAllowedErrorResponse(
              request,
              'Unsupported request: ${request.method}.',
            );
          } else {
            return next(request);
          }
        },
      ),
    );
  }

  RequestHandler middleware(
      Response Function(HttpRequest, Function) middleware) {
    middlewares.add(Middleware(middleware));
    return this;
  }

  Response run(HttpRequest request) {
    if (middlewares.isEmpty) {
      return handler(request);
    }

    var i = 0;
    var handle = middlewares[0];

    final next = (req) {
      if (i == (middlewares.length - 1)) {
        return handler(req);
      } else {
        i = i + 1;
        handle = middlewares[i];
        final resp = handle.handler(req, next);
        return resp;
      }
    };
    final response = handle.handler(request, next);
    return response;
  }
}
